@@ -3,7 +3,11 @@
 /* #define DEBUG Don't even think defining it:) */
 
 #include <mhash.h>
-#include <mcrypt.h>
+#ifndef LIBMCRYPT_22
+# include <mcrypt.h>
+#else
+# include <mcrypt-nm.h>
+#endif
 
 #ifndef LIBMCRYPT_22
 # if MCRYPT_API_VERSION < 20000222
@@ -104,11 +108,7 @@
 #define MODE_SIZE 10
 #define IV_SIZE 64
 
-#ifdef LIBMCRYPT_22
-# define DEFAULT_ALGORITHM "twofish-256"
-#else
 # define DEFAULT_ALGORITHM "twofish"
-#endif
 
 #define DEFAULT_MODE "cbc"
 #define DEFAULT_KEYMODE "mcrypt-sha1"
