@@ -1,4 +1,6 @@
 /*
+ *    Copyright (C) 1998,1999,2000 Nikos Mavroyanopoulos
+ *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
  *    the Free Software Foundation; either version 2 of the License, or
@@ -48,7 +50,7 @@ extern int hash_algorithm;
 char *my_getpass(char *prt)
 {
 	char *atmp;
-	char *ztmp = secure_xmalloc(MAX_KEY_LEN);
+	char *ztmp = secure_mcrypt_malloc(MAX_KEY_LEN);
 	char *btmp;
 	char string_tmp[200];
 
@@ -59,7 +61,7 @@ char *my_getpass(char *prt)
 	strncpy(ztmp, atmp, MAX_KEY_LEN - 1);
 
 #ifndef NO_GETPASS
-	secure_xfree(atmp, strlen(atmp));
+	secure_mcrypt_free(atmp, strlen(atmp));
 #else
 	Bzero(atmp, strlen(atmp));
 #endif
@@ -74,15 +76,15 @@ char *my_getpass(char *prt)
 			_
 			("Keywords do not match or they are too long.\n"));
 #ifndef NO_GETPASS
-		secure_xfree(btmp, MAX_KEY_LEN);
+		secure_mcrypt_free(btmp, MAX_KEY_LEN);
 #else
 		Bzero(btmp, strlen(btmp));
 #endif
-		secure_xfree(ztmp, MAX_KEY_LEN);
+		secure_mcrypt_free(ztmp, MAX_KEY_LEN);
 		return NULL;
 	}
 #ifndef NO_GETPASS
-	secure_xfree(btmp, MAX_KEY_LEN);
+	secure_mcrypt_free(btmp, MAX_KEY_LEN);
 #else
 	Bzero(btmp, strlen(btmp));
 #endif
@@ -223,28 +225,6 @@ int check_file_head(FILE * fstream, char *algorithm, char *mode,
 #else
 		*keysize = keylen;
 #endif
-#ifdef LIBMCRYPT_22
-		if (strstr(algorithm, "twofish") != NULL
-		    || strstr(algorithm, "rc6") != NULL
-		    || strstr(algorithm, "blowfish") != NULL
-		    || strstr(algorithm, "rijndael") != NULL
-		    || strstr(algorithm, "rc2") != NULL
-		    || strstr(algorithm, "serpent") != NULL) {
-
-			sprintf(tmp_buf, "-%u", *keysize * 8);
-			strcat(algorithm, tmp_buf);
-		}
-
-		if (strstr(algorithm, "safer-sk64") != NULL) {
-			strcpy(algorithm, "safer-64");
-		}
-		if (strstr(algorithm, "arcfour") != NULL) {
-			strcpy(algorithm, "rc4");
-		}
-		if (strstr(algorithm, "safer-sk128") != NULL) {
-			strcpy(algorithm, "safer-128");
-		}
-#endif
 
 		read_until_null(tmp_buf, fstream);
 		strcpy(mode, tmp_buf);
@@ -297,7 +277,7 @@ void *read_iv(FILE * fstream, int ivsize)
 		fstream = (FILE *) stdin;
 	}
 
-	IV = xmalloc(ivsize);
+	IV = mcrypt_malloc(ivsize);
 	fread(IV, 1, ivsize, fstream);
 
 	return IV;
@@ -326,7 +306,7 @@ int write_file_head(FILE * filedes, char *algorithm, char *mode,
 	char algo[70];
 #endif
 
-	buf = xmalloc(4);
+	buf = mcrypt_malloc(4);
 
 	buf[0] = '\0';
 	buf[1] = 'm';
@@ -341,38 +321,12 @@ int write_file_head(FILE * filedes, char *algorithm, char *mode,
 	if (fwrite(buf, 1, 4, filedes) != 4) {
 		return 1;
 	}
-	xfree(buf);
+	mcrypt_free(buf);
 
-#ifdef LIBMCRYPT_22
-	if (strstr(algorithm, "cast") == NULL
-	    && strstr(algorithm, "safer-") == NULL) {
-		index = rindex(algorithm, '-');
-		if (index != NULL) {
-			*index = '\0';
-		}
-	}
-	if (strstr(algorithm, "rc4") != NULL) {
-		strcpy(algo, "arcfour");
-		algorithm = algo;
-	}
-	if (strstr(algorithm, "safer-") != NULL) {
-		strcpy(algo, "safer-sk");
-		index = rindex(algorithm, '-');
-		if (index != NULL) {
-			strcat(algo, ++index);
-		}
-		algorithm = algo;
-	}
-#endif
 	if (fwrite(algorithm, 1, strlen(algorithm), filedes) !=
 	    strlen(algorithm)) {
 		return 1;
 	}
-#ifdef LIBMCRYPT_22
-	if (strstr(algorithm, "rijndael") != NULL) {
-		fwrite("-128", 1, 4, filedes);
-	}
-#endif
 	fwrite(&null, 1, 1, filedes);
 
 #ifdef WORDS_BIGENDIAN
@@ -410,7 +364,7 @@ int write_file_head(FILE * filedes, char *algorithm, char *mode,
 	if (fwrite(buf, 1, strlen(buf), filedes) != strlen(buf)) {
 		return 1;
 	}
-	xfree(buf);
+	mcrypt_free(buf);
 	fwrite(&null, 1, 1, filedes);
 
 
@@ -424,7 +378,7 @@ int write_iv(FILE * filedes, void *IV, int ivsize)
 	unsigned char *buf = NULL;
 
 	if (ivsize > 0) {
-		buf = xmalloc(ivsize);
+		buf = mcrypt_malloc(ivsize);
 		if (IV != NULL) {
 			Bzero(buf, ivsize);
 			memmove(buf, IV, ivsize);
@@ -436,7 +390,7 @@ int write_iv(FILE * filedes, void *IV, int ivsize)
 		}
 	}
 
-	xfree(buf);
+	mcrypt_free(buf);
 	return 0;
 
 }
@@ -534,8 +488,8 @@ char **read_key_file(char *file, int *num)
 	}
 	while (fgets(keyword, MAX_KEY_LEN, FROMF) != NULL) {
 		x++;
-		keys = realloc(keys, x * sizeof(char *));
-		keys[x - 1] = xmalloc(strlen(keyword) + 1);
+		keys = mcrypt_realloc(keys, x * sizeof(char *));
+		keys[x - 1] = mcrypt_malloc(strlen(keyword) + 1);
 /* Remove newline */
 		if (keyword[strlen(keyword) - 1] == '\n')
 			keyword[strlen(keyword) - 1] = '\0';
@@ -560,11 +514,11 @@ char *get_cfile(int uid, char *cfile)
 	pwd = getpwuid(uid);
 
 	if (pwd != NULL) {
-		home = xmalloc(strlen(pwd->pw_dir) + strlen(cfile) + 2);
+		home = mcrypt_malloc(strlen(pwd->pw_dir) + strlen(cfile) + 2);
 		strcpy(home, pwd->pw_dir);
 		strcat(home, "/");
 	} else {
-		home = xcalloc(1, strlen(cfile) + 2);
+		home = mcrypt_calloc(1, strlen(cfile) + 2);
 	}
 	strcat(home, cfile);
 
