@@ -72,7 +72,7 @@ char *
  readpass(FILE * fp)
 {
 	static char asterix[MAXLEN + 1];
-        char *input=secure_xmalloc(MAXLEN + 1);
+        char *input=secure_mcrypt_malloc(MAXLEN + 1);
 	char *cp, *ap, c;
 	int i;
 
@@ -134,7 +134,7 @@ char *
 char *
  readpass_old(FILE * fp)
 {
-	char* input=secure_xmalloc(MAXLEN + 1);
+	char* input=secure_mcrypt_malloc(MAXLEN + 1);
 	char *cp;
 
 	fflush(stderr);
