@@ -1,4 +1,6 @@
 /*
+ *    Copyright (C) 1998,1999,2000 Nikos Mavroyanopoulos
+ *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
  *    the Free Software Foundation; either version 2 of the License, or
@@ -14,7 +16,7 @@
  *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: keys.c,v 1.12 2000/03/08 11:41:51 nikos Exp $ */
+/* $Id: keys.c,v 1.20 2000/04/06 11:36:51 nikos Exp $ */
 
 #ifndef DEFINES_H
 #define DEFINES_H
@@ -22,16 +24,20 @@
 #endif
 #include <extra.h>
 #include <xmalloc.h>
+#include <errors.h>
+#include <ufc_crypt.h>
 
-static char rcsid[] = "$Id: keys.c,v 1.12 2000/03/08 11:41:51 nikos Exp $";
-
+static char rcsid[] = "$Id: keys.c,v 1.20 2000/04/06 11:36:51 nikos Exp $";
 
 int mcrypt_gen_key ( char* keymode, void* keyword, int keysize, 
 	void* salt, int saltsize, word8* password, int plen) {
 
+	KEYGEN data;
 	keygenid id=0;
 	hashid algo=0;
+	int icrypt=0;
 
+	if (strcasecmp(keymode, "scrypt")==0) icrypt=1;	
 	if (strcasecmp(keymode, "asis")==0) id=KEYGEN_ASIS;
 	if (strcasecmp(keymode, "hex")==0) id=KEYGEN_HEX;
 	if (strcasecmp(keymode, "pkdes")==0) id=KEYGEN_PKDES;
@@ -43,8 +49,16 @@ int mcrypt_gen_key ( char* keymode, void* keyword, int keysize,
 	if (strcasecmp(keymode, "s2k-salted-sha1")==0) {id=KEYGEN_S2K_SALTED; algo=MHASH_SHA1;}
 	if (strcasecmp(keymode, "s2k-isalted-md5")==0) {id=KEYGEN_S2K_ISALTED; algo=MHASH_MD5;}
 	if (strcasecmp(keymode, "s2k-isalted-sha1")==0) {id=KEYGEN_S2K_ISALTED; algo=MHASH_SHA1;}
-	
-	return mhash_keygen( id, algo, 0,keyword, keysize, salt, saltsize, password, plen);
+
+	if (icrypt==0) {
+		data.hash_algorithm[0] = algo;
+		data.count = 0;
+		data.salt = salt;
+		data.salt_size = saltsize;
+		return mhash_keygen_ext( id, data, keyword, keysize, password, plen);
+	} else {
+		return gen_crypt( keyword, keysize, password, plen);
+	}
 }
 
 
@@ -55,8 +69,8 @@ void *
  fixkey(char *key, unsigned int *len, char* keymode, int keysize,
      int quiet, int stream_flag, void *salt, int salt_size, int enc_mode)
 {
-#ifdef DEBUG
 	int i;
+#ifdef DEBUG
 	unsigned char* psalt=salt;
 #endif
 	char *tmp;
@@ -64,8 +78,8 @@ void *
 	char *tmpkey;
 
 
-        tmpkey= xcalloc(1, MAX_KEY_LEN);
-	keyword = xcalloc(1, keysize);
+        tmpkey= mcrypt_calloc(1, MAX_KEY_LEN);
+	keyword = mcrypt_calloc(1, keysize);
 
 	if (key == NULL) {	/* key was not specified in the command line */
 		tmp = get_password(keysize, enc_mode, len, salt);
@@ -77,7 +91,7 @@ void *
 		memmove(tmpkey, tmp, *len);
 
 #ifndef NO_GETPASS
-		mxfree(tmp, *len);
+		secure_mcrypt_free(tmp, *len);
 #else
 		Bzero(tmp, strlen(tmp));
 #endif
@@ -89,8 +103,11 @@ void *
 
 		/* Generate key by hashing the passphrase */
 		if (salt_size==0) salt=NULL;
-		mcrypt_gen_key( keymode, keyword, keysize, salt, salt_size, 
+		i = mcrypt_gen_key( keymode, keyword, keysize, salt, salt_size,
 					(void *) tmpkey, *len);
+		/* unexpected error in libmhash */
+		if (i<0) err_quit(_("Key transformation failed.\n"));
+		
 		Bzero(tmpkey, *len);	/* key is not bzero'd because it may be used again */
 
 		*len = keysize;
@@ -116,26 +133,26 @@ void *
 #endif
 
 	Bzero( tmpkey, MAX_KEY_LEN);
-	xfree( tmpkey);
+	mcrypt_free( tmpkey);
 	
 	return keyword;
 
 }
 
+int _which_algo ( char* keymode) {
 
-keygenid _which_algo ( char* keymode) {
-
-	if (strcasecmp(keymode, "asis")==0) return KEYGEN_ASIS;
-	if (strcasecmp(keymode, "hex")==0) return KEYGEN_HEX;
-	if (strcasecmp(keymode, "pkdes")==0) return KEYGEN_PKDES;
-	if (strcasecmp(keymode, "mcrypt-sha1")==0) return KEYGEN_MCRYPT;
-	if (strcasecmp(keymode, "mcrypt-md5")==0) return KEYGEN_ASIS;
-	if (strcasecmp(keymode, "s2k-simple-md5")==0) return KEYGEN_S2K_SIMPLE;
-	if (strcasecmp(keymode, "s2k-simple-sha1")==0) return KEYGEN_S2K_SIMPLE;
-	if (strcasecmp(keymode, "s2k-salted-md5")==0) return KEYGEN_S2K_SALTED;
-	if (strcasecmp(keymode, "s2k-salted-sha1")==0) return KEYGEN_S2K_SALTED;
-	if (strcasecmp(keymode, "s2k-isalted-md5")==0) return KEYGEN_S2K_ISALTED;
-	if (strcasecmp(keymode, "s2k-isalted-sha1")==0) return KEYGEN_S2K_ISALTED;
+	if (strcasecmp(keymode, "asis")==0) return 0;
+	if (strcasecmp(keymode, "scrypt")==0) return 0;
+	if (strcasecmp(keymode, "hex")==0) return 0;
+	if (strcasecmp(keymode, "pkdes")==0) return 0;
+	if (strcasecmp(keymode, "mcrypt-sha1")==0) return 0;
+	if (strcasecmp(keymode, "mcrypt-md5")==0) return 0;
+	if (strcasecmp(keymode, "s2k-simple-md5")==0) return 0;
+	if (strcasecmp(keymode, "s2k-simple-sha1")==0) return 0;
+	if (strcasecmp(keymode, "s2k-salted-md5")==0) return 0;
+	if (strcasecmp(keymode, "s2k-salted-sha1")==0) return 0;
+	if (strcasecmp(keymode, "s2k-isalted-md5")==0) return 0;
+	if (strcasecmp(keymode, "s2k-isalted-sha1")==0) return 0;
 
 	return -1;
 }
