@@ -1,10 +1,5 @@
-/*  Mini-Crypt 2.2 program. It encrypts text or binary files using
- *  keywords. The keyword is NOT included in the encrypted file so
- *  there is no way to decrypt it, if you forgot it. 
- *  For a brief description of the algorithms read the README file and
- *  the man page.
- *                                     Nikos Mavroyanopoulos
- *                                         nmav@i-net.paiko.gr
+/*  mcrypt 2.5 - encrypts text or binary files using symmetric algorithms.
+ *  For a brief description of the algorithms read the man page.
  *
  *    Copyright (C) 1998,1999,2000 Nikos Mavroyanopoulos
  *
@@ -24,7 +19,7 @@
  */
 
 
-/* $Id: mcrypt.c,v 1.25 2000/03/10 19:19:12 nikos Exp $ */
+/* $Id: mcrypt.c,v 1.36 2000/04/02 18:26:18 nikos Exp $ */
 
 #ifndef DEFINES_H
 #define DEFINES_H
@@ -42,7 +37,7 @@
 
 
 static char rcsid[] =
-    "$Id: mcrypt.c,v 1.25 2000/03/10 19:19:12 nikos Exp $";
+    "$Id: mcrypt.c,v 1.36 2000/04/02 18:26:18 nikos Exp $";
 
 char tmperr[128];
 unsigned int stream_flag = FALSE;
@@ -54,7 +49,7 @@ int keysize;
 int nodelete = FALSE;		/* Delete by default */
 int nolock = FALSE;		/* Use locks by default */
 int noiv = FALSE;
-int hash_algorithm = MHASH_MD5;
+int hash_algorithm = MHASH_CRC32;
 int noecho = TRUE;		/* Echo asterisks by default */
 int double_check = FALSE;	/* Do not double check for passwords when decrypting */
 int quiet = TRUE;		/* silent by default */
@@ -148,7 +143,7 @@ void usage(void)
 			  " -h --help                    prints this help\n"
 			  " -v --version                 prints the version number\n"
 			  "\n"
-			  "Report bugs to mcrypt@i-net.paiko.gr.\n\n"));
+			  "Report bugs to mcrypt-dev@argeas.cs-net.gr.\n\n"));
 }
 
 void mcrypt_license()
@@ -178,10 +173,10 @@ int check_hash_algo(char *chain)
 		if (y != NULL) {
 			_tolow(y, strlen(y));
 			if (strcmp(y, chain) == 0) {
-				xfree(y);
+				mcrypt_free(y);
 				return i;
 			}
-			xfree(y);
+			mcrypt_free(y);
 		}
 	}
 	return -1;
@@ -195,13 +190,13 @@ int check_algo(char *chain)
 	char *y;
 
 	for (i = 0; i < 255; i++) {
-		if (is_ok_algorithm(i) == 0) {
+		if (mcrypt_is_ok_algorithm(i) == 0) {
 			y = mcrypt_get_algorithms_name(i);
 			if (strcmp(y, chain) == 0) {
-				xfree(y);
+				mcrypt_free(y);
 				return i;
 			}
-			xfree(y);
+			mcrypt_free(y);
 		}
 	}
 	return -1;
@@ -213,13 +208,13 @@ int check_mode(char *chain)
 	char *y;
 
 	for (i = 0; i < 255; i++) {
-		if (is_ok_mode(i) == 0) {
+		if (mcrypt_is_ok_mode(i) == 0) {
 			y = mcrypt_get_modes_name(i);
 			if (strcmp(y, chain) == 0) {
-				xfree(y);
+				mcrypt_free(y);
 				return i;
 			}
-			xfree(y);
+			mcrypt_free(y);
 		}
 	}
 	return -1;
@@ -246,7 +241,10 @@ int main(int argc, char **argv)
 	short int ein = 0, din = 0, kin = 0, force = 0, return_val = 0;
 	char *einfile = 0, *dinfile = 0, *keyword = 0;
 	char **file, **keyfile = NULL, *cfile;
-	int x, y, i, file_count = 0, keys = 0, used_algo = FALSE, v, *siz;
+	int x, y, i, file_count = 0, keys = 0, used_algo = FALSE;
+#ifndef LIBMCRYPT_22
+	int *siz;
+#endif
 	gaainfo info;
 
 #ifdef HAVE_SIGNAL_H
@@ -267,13 +265,11 @@ int main(int argc, char **argv)
 	textdomain(PACKAGE);
 #endif
 
-
-
-	if ((v = gaa(argc, argv, &info)) != -1) {
+	if (( gaa(argc, argv, &info)) != -1) {
 		err_quit(_("Error in the arguments.\n"));
 	}
 	if (info.config_file != NULL) {
-		cfile = xmalloc(strlen(info.config_file));
+		cfile = mcrypt_malloc(strlen(info.config_file));
 		strcpy(cfile, info.config_file);
 		if (check_file(cfile) != 1) {
 			err_crit(_
@@ -283,34 +279,34 @@ int main(int argc, char **argv)
 #ifdef HAVE_GETPWUID
 		cfile = get_cfile(getuid(), ".mcryptrc");
 #else
-		cfile = xmalloc(11);
+		cfile = mcrypt_malloc(11);
 		strcpy(cfile, ".mcryptrc");
 #endif
 	}
 
 	/* If config file exists */
 	if (check_file(cfile) == 1) {
-		if ((v = gaa_file(cfile, &info)) != -1) {
+		if (( gaa_file(cfile, &info)) != -1) {
 			/* gets options from file 'config' */
 			err_quit(_("Error in the configuration file.\n"));
 		}
 	}
-	xfree(cfile);
+	mcrypt_free(cfile);
 
 
 	if (check_env() == TRUE) {
 
 		if (get_env_key() != NULL) {
 			info.keylen = 1;
-			xfree(info.keys);
+			mcrypt_free(info.keys);
 			info.keys = get_env_key();
 		}
 		if (get_env_algo() != NULL) {
-			xfree(info.algorithm);
+			mcrypt_free(info.algorithm);
 			info.algorithm = get_env_algo();
 		}
 		if (get_env_mode() != NULL) {
-			xfree(info.mode);
+			mcrypt_free(info.mode);
 			info.mode = get_env_mode();
 		}
 		if (get_env_bit_mode() != 0) {
@@ -321,7 +317,7 @@ int main(int argc, char **argv)
  * This will be uncommented when gaa is corrected.
  */
 
-	if ((v = gaa(argc, argv, &info)) != -1) {
+	if ( gaa(argc, argv, &info) != -1) {
 		err_quit(_("Error in the arguments.\n"));
 	}
 /* Examine how we were called */
@@ -331,7 +327,7 @@ int main(int argc, char **argv)
 		din = TRUE;
 		ein = FALSE;
 	} else {
-		if (strlen(argv[0]) > 5) {
+		if (strlen(argv[0]) > 6) {
 			Bzero(argv[0], strlen(argv[0]));
 			strcpy(argv[0], "mcrypt");
 		}
@@ -342,7 +338,11 @@ int main(int argc, char **argv)
 
 	/* File pointers are as much as the file arguments */
 
-	file = xmalloc((info.size) * sizeof(char *));
+	if ((info.size) * sizeof(char *) > 0) {
+		file = mcrypt_malloc((info.size) * sizeof(char *));
+	} else {
+		file = NULL;
+	}
 
 	if (info.ed_specified != 0) {
 		din = info.din;
@@ -385,7 +385,7 @@ int main(int argc, char **argv)
 	}
 
 	if (info.hash != NULL) {
-		hash = xmalloc(strlen(info.hash));
+		hash = mcrypt_malloc(strlen(info.hash));
 		strcpy(hash, info.hash);
 		hash_algorithm = check_hash_algo(hash);
 
@@ -401,7 +401,8 @@ int main(int argc, char **argv)
 			exit(1);
 		}
 	} else {
-		hash = xmalloc(strlen("crc32") + 1);
+		hash_algorithm = MHASH_CRC32;
+		hash = mcrypt_malloc(strlen("crc32") + 1);
 		strcpy(hash, "crc32");
 	}
 
@@ -415,12 +416,12 @@ int main(int argc, char **argv)
 	} else {
 		if (info.keys != NULL) {
 			kin = 2;
-			keyfile = xmalloc(info.keylen * sizeof(char *));
+			keyfile = mcrypt_malloc(info.keylen * sizeof(char *));
 			keys = info.keylen;
 
 			for (i = 0; i < info.keylen; i++) {
 				keyfile[i] =
-				    secure_xmalloc(strlen(info.keys[i]) +
+				    secure_mcrypt_malloc(strlen(info.keys[i]) +
 						   10);
 				strcpy(keyfile[i], info.keys[i]);
 			}
@@ -428,17 +429,20 @@ int main(int argc, char **argv)
 	}
 
 	if (info.mode != NULL) {
-		mode = xmalloc(strlen(info.mode) + 1);
+		mode = mcrypt_malloc(strlen(info.mode) + 1);
 		strcpy(mode, info.mode);
 	} else {
 		mode = strdup(DEFAULT_MODE);
 	}
+	
 	if (info.algorithm != NULL) {
 		used_algo = TRUE;
-		algorithm = xmalloc(strlen(info.algorithm) + 1);
+		algorithm = mcrypt_malloc(strlen(info.algorithm) + 1);
 		strcpy(algorithm, info.algorithm);
+
 #ifndef LIBMCRYPT_22
-		i = mcrypt_is_block_algorithm(algorithm, algorithms_directory);
+
+		i = mcrypt_module_is_block_algorithm(algorithm, algorithms_directory);
 
 		if (i < 0) {
 			fprintf(stderr, _("Error in algorithm '%s'.\n"),
@@ -447,10 +451,10 @@ int main(int argc, char **argv)
 		}
 
 		if (i == 0) {
-			y = mcrypt_is_block_algorithm_mode(mode, modes_directory);
+			y = mcrypt_module_is_block_algorithm_mode(mode, modes_directory);
 			if (y >= 0) {
 				mode =
-				    xrealloc(mode, strlen("stream") + 1);
+				    mcrypt_realloc(mode, strlen("stream") + 1);
 				strcpy(mode, "stream");
 			} else {
 				fprintf(stderr, _("Error in mode '%s'.\n"),
@@ -460,6 +464,7 @@ int main(int argc, char **argv)
 		}
 
 #else
+
 		i = check_algo(algorithm);
 		if (i < 0) {
 			fprintf(stderr, _("Error in algorithm '%s'.\n"),
@@ -471,10 +476,10 @@ int main(int argc, char **argv)
 			fprintf(stderr, _("Error in mode '%s'.\n"), mode);
 			exit(1);
 		}
-		if (is_block_algorithm(i) == 0) {
-			if (is_block_mode(y) != 0) {
+		if (mcrypt_is_block_algorithm(i) == 0) {
+			if (mcrypt_is_block_algorithm_mode(y) != 0) {
 				mode =
-				    xrealloc(mode, strlen("stream") + 1);
+				    mcrypt_realloc(mode, strlen("stream") + 1);
 				strcpy(mode, "stream");
 			}
 		}
@@ -485,7 +490,7 @@ int main(int argc, char **argv)
 
 	if (info.mode != NULL) {
 #ifndef LIBMCRYPT_22
-		y = mcrypt_is_block_algorithm_mode(mode, modes_directory);
+		y = mcrypt_module_is_block_algorithm_mode(mode, modes_directory);
 		if (y < 0) {
 			fprintf(stderr, _("Error in mode '%s'.\n"), mode);
 			exit(1);
@@ -500,6 +505,7 @@ int main(int argc, char **argv)
 	}
 
 #ifdef LIBMCRYPT_22
+
 	i = check_algo(algorithm);
 	i = keysize = mcrypt_get_key_size(i);
 
@@ -507,11 +513,11 @@ int main(int argc, char **argv)
 
 	if (info.keysize != 0) {
 #ifndef LIBMCRYPT_22
-		if (info.keysize > mcrypt_get_algo_key_size(algorithm, algorithms_directory)) {
+		if (info.keysize > mcrypt_module_get_algo_key_size(algorithm, algorithms_directory)) {
 			err_quit(_("The specified key size is too large for this algorithm.\n"));
 		}
 
-		siz = mcrypt_get_algo_supported_key_sizes( algorithm, algorithms_directory, &i);
+		siz = mcrypt_module_get_algo_supported_key_sizes( algorithm, algorithms_directory, &i);
 		x=0;
 		for (y=0;y<i;y++) {
 			if (siz[y]==info.keysize) {
@@ -550,6 +556,10 @@ int main(int argc, char **argv)
 
 /* ok now how many files were specified? */
 	i = 0;
+
+	/* '-' is used to refer to stdin */
+	if (info.size==1 && strcmp(info.input[0], "-")==0) info.size-=1;
+
 	file_count += (info.size);
 
 
@@ -570,7 +580,7 @@ int main(int argc, char **argv)
 
 		if (i != 0) {
 			if (outfile != NULL)
-				xfree(outfile);
+				mcrypt_free(outfile);
 		}
 
 		/* If keyword file specified choose the i-th keyword */
@@ -578,7 +588,7 @@ int main(int argc, char **argv)
 		if (kin == 2 && i <= (keys - 1)) {
 			keyword = keyfile[i];
 			if (i != 0) {
-				secure_xfree(keyfile[i - 1],
+				secure_mcrypt_free(keyfile[i - 1],
 					     strlen(keyfile[i - 1]));	/* Free previous keyword */
 			}
 		}
@@ -647,7 +657,7 @@ int main(int argc, char **argv)
 				     && dinfile[y - 2] == 'n'
 				     && dinfile[y - 3] == '.')) {
 
-					outfile = xcalloc(y - 2, 1);
+					outfile = mcrypt_calloc(y - 2, 1);
 					strncpy(outfile, dinfile, y - 3);
 #ifdef HAVE_STAT
 					/* But if it exists exit */
@@ -667,7 +677,7 @@ int main(int argc, char **argv)
 				} else {	/* append .dc */
 
 					cleanDelete = TRUE;
-					outfile = xcalloc(y + 5, 1);
+					outfile = mcrypt_calloc(y + 5, 1);
 					strncpy(outfile, dinfile, y);
 					strcat(outfile, ".dc");
 
@@ -675,7 +685,7 @@ int main(int argc, char **argv)
 				}
 			} else {	/* encryption- append .nc */
 				outfile =
-				    xcalloc(strlen(einfile) + 5 + 4, 1);
+				    mcrypt_calloc(strlen(einfile) + 5 + 4, 1);
 				strcpy(outfile, einfile);
 				/* if file has already the .nc ignore it */
 				if (strstr(outfile, ".nc") != NULL) {
@@ -713,7 +723,7 @@ int main(int argc, char **argv)
 
 			}
 		} else {	/* if streams */
-			outfile = xmalloc(7);
+			outfile = mcrypt_malloc(7);
 			strcpy(outfile, "stdout");
 		}
 
@@ -830,9 +840,10 @@ int main(int argc, char **argv)
 	}			/* the main loop */
 
 /* Clear the last keyword used */
-	if (keyword != NULL)
+	if (keyword != NULL) {
 		Bzero(keyword, strlen(keyword));
-	xfree(keyword);
+		mcrypt_free(keyword);
+	}
 
 /* Clear ALL arguments in the command line */
 	for (i = 0; i < argc; i++) {
@@ -870,14 +881,17 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	word32 *keyword;
 #ifdef LIBMCRYPT_22
 	int algo, _mode;
+#else
+	int tc;
 #endif
-	hashid tm = 0;
+	MHASH tm = 0;
 	int i = 0, how = 0;
 	unsigned int len = 0;
 	FILE *FROMF;
 	int keylen, salt_size;
 	int blocksize;
-	int td, tc, pad_size;
+	MCRYPT td;
+	int pad_size;
 	char *command = NULL;
 	keygenid ki;
 	word32 *IV = NULL;
@@ -892,8 +906,8 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	    mcrypt_module_open(algorithm, algorithms_directory, mode,
 			       modes_directory);
 
-	if (td < 0) {
-		mcrypt_perror(td);
+	if (td == MCRYPT_FAILED) {
+		err_crit(_("Mcrypt failed to open module.\n"));
 		return 1;
 	}
 #else
@@ -906,10 +920,10 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 #ifndef LIBMCRYPT_22
 	if (keysize == 0)
-		keysize = mcrypt_get_key_size(td);
+		keysize = mcrypt_enc_get_key_size(td);
 
 	keylen = keysize;
-	blocksize = mcrypt_get_block_size(td);
+	blocksize = mcrypt_enc_get_block_size(td);
 #else
 	if (keysize == 0)
 		keysize = mcrypt_get_key_size(algo);
@@ -919,10 +933,10 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 #endif
 
 
-	ciphertext = xmalloc(blocksize * BUFFER_SIZE);
+	ciphertext = mcrypt_malloc(blocksize * BUFFER_SIZE);
 
 	if (bare_flag == FALSE) {
-		salt = xmalloc(SALT_SIZE);	/* 20 bytes salt */
+		salt = mcrypt_malloc(SALT_SIZE);	/* 20 bytes salt */
 
 		/* Fill the salt with random data */
 		for (i = 0; i < SALT_SIZE / sizeof(word32); i++) {
@@ -935,11 +949,11 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		}
 	}
 #ifndef LIBMCRYPT_22
-	IV = secure_xmalloc(mcrypt_get_iv_size(td));
+	IV = secure_mcrypt_malloc(mcrypt_enc_get_iv_size(td));
 /* Random number generator */
-	for (i = 0; i < mcrypt_get_iv_size(td) / sizeof(word32); i++) {
+	for (i = 0; i < mcrypt_enc_get_iv_size(td) / sizeof(word32); i++) {
 #else
-	IV = secure_xmalloc(mcrypt_get_block_size(algo));
+	IV = secure_mcrypt_malloc(mcrypt_get_block_size(algo));
 /* Random number generator */
 	for (i = 0; i < mcrypt_get_block_size(algo) / sizeof(word32); i++) {
 #endif
@@ -1039,8 +1053,8 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 				salt, salt_size);
 	}
 #ifndef LIBMCRYPT_22
-	if (mcrypt_mode_has_iv(td) == 1 && noiv == FALSE) {
-		write_iv(TOF, IV, mcrypt_get_iv_size(td));
+	if (mcrypt_enc_mode_has_iv(td) == 1 && noiv == FALSE) {
+		write_iv(TOF, IV, mcrypt_enc_get_iv_size(td));
 	}
 #else
 	if (mcrypt_mode_has_iv(_mode) == 1 && noiv == FALSE) {
@@ -1084,7 +1098,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	}
 #else
 	td = mcrypt_generic_init(_mode, algo, keyword, len, IV);
-	if (td < 0) {
+	if (td == MCRYPT_FAILED) {
 		return 1;
 	}
 #endif
@@ -1106,7 +1120,11 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 			if (bare_flag == FALSE)
 				mhash(tm, ciphertext, how);
 
-			if (is_block_mode(td) == 1) {
+#ifndef LIBMCRYPT_22
+			if (mcrypt_enc_is_block_mode(td) == 1) {
+#else
+			if (mcrypt_is_block_mode(_mode) == 1) {
+#endif
 				rest_blocks = how % blocksize;
 				full_blocks = how / blocksize;
 
@@ -1152,20 +1170,32 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		    make_mult(rest_blocks + hash_size + 1, blocksize);
 
 		/* in case of a stream cipher no padding is needed */
-		if (is_block_mode(td) == 0) {
+#ifndef LIBMCRYPT_22
+		if (mcrypt_enc_is_block_mode(td) == 0) {
+#else
+		if (mcrypt_is_block_mode(_mode) == 0) {
+#endif
 			pad_size = hash_size;
 		}
 	} else {		/* bare_flag==TRUE */
 		pad_size = blocksize;
-		if (is_block_mode(td) == 0) {
+#ifndef LIBMCRYPT_22
+		if (mcrypt_enc_is_block_mode(td) == 0) {
+#else
+		if (mcrypt_is_block_mode(_mode) == 0) {
+#endif
 			pad_size = 0;
 		}
 	}
 
 	if (pad_size > 0)
-		ciphertext_pad = xmalloc(pad_size);
+		ciphertext_pad = mcrypt_malloc(pad_size);
 
-	if (is_block_mode(td) == 1) {
+#ifndef LIBMCRYPT_22
+	if (mcrypt_enc_is_block_mode(td) == 1) {
+#else
+	if (mcrypt_is_block_mode(_mode) == 1) {
+#endif
 		memmove(ciphertext_pad,
 			&ciphertext[full_blocks * blocksize], rest_blocks);
 		if (bare_flag == FALSE) {
@@ -1228,11 +1258,11 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	mcrypt_generic_end(td);
 
 	if (bare_flag == FALSE) {
-		xfree(salt);
+		mcrypt_free(salt);
 	}
 
-	secure_xfree(keyword, keylen);
-	xfree(ciphertext);
+	secure_mcrypt_free(keyword, keylen);
+	mcrypt_free(ciphertext);
 
 
 	return 0;
@@ -1246,12 +1276,13 @@ int
 decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 {
 	char *fcrc32, *newcrc32;
-	hashid tm = 0;
+	MHASH tm = 0;
 	char tmp_buf[BUFFER_SIZE];
 	int how = 0;
-	int i = 0, j;
+	int i = 0;
 	int blocksize, crcsize, salt_size;
-	int td, pid, buf_block, start;
+	MCRYPT td;
+	int pid, buf_block, start;
 	word32 *IV = NULL;
 	word8 *ciphertext;
 	word8 *ciphertext_old, *ciphertext_p_old;
@@ -1265,6 +1296,8 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	int hash_size, pad_size;
 #ifdef LIBMCRYPT_22
 	int algo, _mode;
+#else
+	int j;
 #endif
 	FILE *RTOF;
 	FILE *FROMF;
@@ -1295,20 +1328,20 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 
 	if (bare_flag == FALSE) {
-		salt = xmalloc(SALT_SIZE);
+		salt = mcrypt_malloc(SALT_SIZE);
 		if (check_file_head
 		    (FROMF, local_algorithm, local_mode, local_keymode,
 		     &keysize, local_salt, &salt_size) != 0) {
 			err_crit(_("No valid file headers found.\n"));
 			return 1;
 		}
-		salt = xmalloc(salt_size);
+		salt = mcrypt_malloc(salt_size);
 		memmove(salt, local_salt, salt_size);
-		algorithm = xmalloc(strlen(local_algorithm)+1);
+		algorithm = mcrypt_malloc(strlen(local_algorithm)+1);
 		strcpy(algorithm, local_algorithm);
-		mode = xmalloc(strlen(local_mode)+1);
+		mode = mcrypt_malloc(strlen(local_mode)+1);
 		strcpy(mode, local_mode);
-		keymode = xmalloc(strlen(local_keymode)+1);
+		keymode = mcrypt_malloc(strlen(local_keymode)+1);
 		strcpy(keymode, local_keymode);
 	} else {
 		salt = NULL;
@@ -1331,21 +1364,21 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	td =
 	    mcrypt_module_open(algorithm, algorithms_directory, mode,
 			       modes_directory);
-	if (td < 0) {
-		mcrypt_perror(td);
+	if (td == MCRYPT_FAILED) {
+		err_crit(_("Mcrypt failed to open module.\n"));
 		return 1;
 	}
 
 	if (keysize == 0)
-		keysize = mcrypt_get_key_size(td);
+		keysize = mcrypt_enc_get_key_size(td);
 
 
-	blocksize = mcrypt_get_block_size(td);
+	blocksize = mcrypt_enc_get_block_size(td);
 
 #endif
 
-	ciphertext = xmalloc(blocksize * BUFFER_SIZE);
-	ciphertext_old = xmalloc(blocksize);
+	ciphertext = mcrypt_malloc(blocksize * BUFFER_SIZE);
+	ciphertext_old = mcrypt_malloc(blocksize);
 
 	if (quiet == FALSE)
 		fprintf(stderr,
@@ -1375,11 +1408,11 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		return 1;
 
 #ifndef LIBMCRYPT_22
-	if (mcrypt_mode_has_iv(td) == 1) {
+	if (mcrypt_enc_mode_has_iv(td) == 1) {
 		if (noiv == FALSE) {
-			IV = read_iv(FROMF, mcrypt_get_iv_size(td));
+			IV = read_iv(FROMF, mcrypt_enc_get_iv_size(td));
 		} else {
-			IV = xcalloc(1, mcrypt_get_iv_size(td));
+			IV = mcrypt_calloc(1, mcrypt_enc_get_iv_size(td));
 		}
 	} else {
 		IV = NULL;
@@ -1389,7 +1422,7 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		if (noiv == FALSE) {
 			IV = read_iv(FROMF, mcrypt_get_block_size(algo));
 		} else {
-			IV = xcalloc(1, mcrypt_get_block_size(algo));
+			IV = mcrypt_calloc(1, mcrypt_get_block_size(algo));
 		}
 	} else {
 		IV = NULL;
@@ -1412,7 +1445,7 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	}
 #else
 	td = mcrypt_generic_init(_mode, algo, keyword, len, IV);
-	if (td < 0) {
+	if (td == MCRYPT_FAILED) {
 		return 1;
 	}
 #endif
@@ -1422,7 +1455,7 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	}
 
 	crcsize = hash_size;
-	fcrc32 = xmalloc(crcsize);
+	fcrc32 = mcrypt_malloc(crcsize);
 
 	/* decryption starts here */
 
@@ -1430,17 +1463,25 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		pad_size = make_mult(blocksize + hash_size, blocksize);
 
 		/* in case of a stream cipher no padding is needed */
-		if (is_block_mode(td) == 0) {
+#ifndef LIBMCRYPT_22
+		if (mcrypt_enc_is_block_mode(td) == 0) {
+#else
+                if (mcrypt_is_block_mode(_mode) == 0) {
+#endif
 			pad_size = hash_size;
 		}
 
-		ciphertext_p_old = xmalloc(pad_size);
+		ciphertext_p_old = mcrypt_malloc(pad_size);
 	} else {
 		pad_size = 0;
 		ciphertext_p_old = NULL;
 	}
 
-	if (is_block_mode(td) == 1) {
+#ifndef LIBMCRYPT_22
+	if (mcrypt_enc_is_block_mode(td) == 1) {
+#else
+	if (mcrypt_is_block_mode(_mode) == 1) {
+#endif
 		if (bare_flag == FALSE) {
 			how =
 			    fread(ciphertext, 1, blocksize * BUFFER_SIZE,
@@ -1756,20 +1797,24 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 	if (pad_size > 0) {
 		Bzero(ciphertext_p_old, pad_size);
-		xfree(ciphertext_p_old);
+		mcrypt_free(ciphertext_p_old);
 	}
 	Bzero(ciphertext_old, blocksize);
 	Bzero(ciphertext, blocksize * BUFFER_SIZE);
 
-	xfree(ciphertext_old);
-	xfree(ciphertext);
+	mcrypt_free(ciphertext_old);
+	mcrypt_free(ciphertext);
 
 	Bzero(tmp_buf, BUFFER_SIZE);
-	secure_xfree(keyword, keysize);
+	secure_mcrypt_free(keyword, keysize);
 
 	if (bare_flag == FALSE) {
 		newcrc32 = mhash_end(tm);
-		if (is_block_mode(td) == 1) {
+#ifndef LIBMCRYPT_22
+		if (mcrypt_enc_is_block_mode(td) == 1) {
+#else
+                if (mcrypt_is_block_mode(_mode) == 1) {
+#endif
 			if (hash_size > blocksize) {
 				hash_size = blocksize;
 			}
@@ -1854,15 +1899,16 @@ void rol_buf(void *buffer, int buffersize, void *value)
 }
 
 #ifndef LIBMCRYPT_22
-void print_list(void)
+int print_list(void)
 {
-	int td;
+	MCRYPT td;
 	int i, imax;
 	int j, jmax;
 	int start;
 	char **names;
 	char **modes;
 
+
 	names = mcrypt_list_algorithms(algorithms_directory, &jmax);
 	modes = mcrypt_list_modes(modes_directory, &imax);
 
@@ -1876,23 +1922,24 @@ void print_list(void)
 					       algorithms_directory,
 					       modes[i], modes_directory);
 
-			if (td > 0) {
+			if (td != MCRYPT_FAILED) {
 				if (start == 0)
 					printf("(%d): ",
-					       mcrypt_get_key_size(td));
+					       mcrypt_enc_get_key_size(td));
 				printf("%s ", modes[i]);
 				mcrypt_module_close(td);
 				start = 1;
 			}
-
 		}
 		printf("\n");
 	}
+
 	mcrypt_free_p(names, jmax);
 	mcrypt_free_p(modes, imax);
+	return 0;
 }
 #else
-void print_list(void)
+int print_list(void)
 {
 	int tmpi, tmpj;
 	char *tmpc;
@@ -1902,11 +1949,11 @@ void print_list(void)
 		tmpc = mcrypt_get_algorithms_name(tmpi);
 		if (tmpc != NULL) {
 			fprintf(stdout, " %s:", tmpc);
-			xfree(tmpc);
+			mcrypt_free(tmpc);
 
-			if (is_block_algorithm(tmpi) == TRUE) {
+			if (mcrypt_is_block_algorithm(tmpi) == TRUE) {
 				for (tmpj = 0; tmpj < 256; tmpj++) {
-					if (is_block_mode(tmpj) != 0) {
+					if (mcrypt_is_block_algorithm_mode(tmpj) != 0) {
 						tmpc =
 						    mcrypt_get_modes_name
 						    (tmpj);
@@ -1914,14 +1961,14 @@ void print_list(void)
 							fprintf(stdout,
 								" %s",
 								tmpc);
-							xfree(tmpc);
+							mcrypt_free(tmpc);
 						}
 					}
 				}
 				fprintf(stdout, "\n");
 			} else {
 				for (tmpj = 0; tmpj < 256; tmpj++) {
-					if (is_block_mode(tmpj) == 0) {
+					if (mcrypt_is_block_algorithm_mode(tmpj) == 0) {
 						tmpc =
 						    mcrypt_get_modes_name
 						    (tmpj);
@@ -1929,7 +1976,7 @@ void print_list(void)
 							fprintf(stdout,
 								" %s",
 								tmpc);
-							xfree(tmpc);
+							mcrypt_free(tmpc);
 						}
 					}
 				}
@@ -1939,15 +1986,17 @@ void print_list(void)
 		}
 	}
 
+	return 0;
 }
 
 
 #endif
 
-void print_keylist(void)
+int print_keylist(void)
 {
 
 	printf("asis\n");
+	printf("scrypt\n");
 	printf("mcrypt-md5\n");
 	printf("mcrypt-sha1\n");
 	printf("hex\n");
@@ -1958,11 +2007,13 @@ void print_keylist(void)
 	printf("s2k-salted-sha1\n");
 	printf("s2k-isalted-md5\n");
 	printf("s2k-isalted-sha1\n");
+	
+	return 0;
 }
 
-void print_hashlist(void)
+int print_hashlist(void)
 {
-	int tmpi, tmpj;
+	int tmpi;
 	char *tmpc;
 
 	fprintf(stdout, _("Supported Hash Algorithms:\n"));
@@ -1971,7 +2022,8 @@ void print_hashlist(void)
 		if (tmpc != NULL) {
 			_tolow(tmpc, strlen(tmpc));
 			fprintf(stdout, "%s\n", tmpc);
-			xfree(tmpc);
+			mcrypt_free(tmpc);
 		}
 	}
+	return 0;
 }
