@@ -1,4 +1,6 @@
 /*
+ *    Copyright (C) 1998,1999,2000 Nikos Mavroyanopoulos
+ *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
  *    the Free Software Foundation; either version 2 of the License, or
@@ -23,18 +25,11 @@
 
 #include <xmalloc.h>
 
-/* found in libmcrypt */
-extern void* mxmalloc();
-extern void* mxcalloc();
-extern void* mxrealloc();
-extern void mxfree();
-
 static char rcsid[] = "$Id: xmalloc.c,v 1.3 2000/03/07 07:16:18 nikos Exp $";
 
 /* memory allocation */
-
 void *
- xmalloc(size_t size)
+ mcrypt_malloc(size_t size)
 {
 	char *x;
 
@@ -56,7 +51,7 @@ void *
 }
 
 void *
- xcalloc(size_t nmemb, size_t size)
+ mcrypt_calloc(size_t nmemb, size_t size)
 {
 	char *x;
 
@@ -79,7 +74,7 @@ void *
 }
 
 void *
- xrealloc(void *ptr, size_t size)
+ mcrypt_realloc(void *ptr, size_t size)
 {
 	char *x;
 
@@ -104,16 +99,16 @@ void *
 
 
 void *
- secure_xmalloc(size_t size)
+ secure_mcrypt_malloc(size_t size)
 {
 	char *x;
 
-	x = mxmalloc(size);
+	x = malloc(size);
 	if (x != NULL) {
 		return x;
 	} else {
 		sleep(1);
-		x = mxmalloc(size);
+		x = malloc(size);
 		if (x != NULL) {
 			return x;
 		} else {
@@ -126,16 +121,16 @@ void *
 }
 
 void *
- secure_xcalloc(size_t nmemb, size_t size)
+ secure_mcrypt_calloc(size_t nmemb, size_t size)
 {
 	char *x;
 
-	x = mxcalloc(nmemb, size);
+	x = calloc(nmemb, size);
 	if (x != NULL) {
 		return x;
 	} else {
 		sleep(1);
-		x = mxcalloc(nmemb, size);
+		x = calloc(nmemb, size);
 		if (x != NULL) {
 			return x;
 		} else {
@@ -149,16 +144,16 @@ void *
 }
 
 void *
- secure_xrealloc(void *ptr, size_t size)
+ secure_mcrypt_realloc(void *ptr, size_t size)
 {
 	char *x;
 
-	x = mxrealloc(ptr, size);
+	x = realloc(ptr, size);
 	if (x != NULL) {
 		return x;
 	} else {
 		sleep(1);
-		x = mxrealloc(ptr, size);
+		x = realloc(ptr, size);
 		if (x != NULL) {
 			return x;
 		} else {
@@ -171,10 +166,12 @@ void *
 
 }
 
-void xfree( void* ptr) {
+void mcrypt_free( void* ptr) {
 	free(ptr);
 }
 
-void secure_xfree( void* ptr, int size) {
-	mxfree(ptr, size);
+void secure_mcrypt_free( void* ptr, int size) {
+	Bzero(ptr, size);
+	free(ptr);
 }
+
